# Unfortunately, this is a (near) copy of runR() in shinyTexAux.R.
# This is part of compilation, while that is used at run time.
#
# Run code lines in an environment and return the parsed input,
# the output, the error messags, the error code (line(s) that caused
# the error), and whether or not the code had nefarious elements.
# The 'input' and 'output' are corresponding lists with one element
# per parseable unit in the 'code'.
runR = function(code, env, allowSemicolons=FALSE) {
  c0 = character(0)
  if (length(code)==0 || all(code == "")) {
    return(list(input=c0, output=c0, errorMessage=c0, errorCode=c0, nefarious=FALSE))
  }
  blackList = paste(c("file[.]remove", "file[.]rename", "file[.]rename", "file[.]copy",
                      "system2?", "shell", "shell[.]exec", 
                      "[.]Call", "[.]Internal", "[.]Primitive", "[.]External",
                      "dyn[.]load", "[.]C", "[.]Fortran"), 
                    "[[:space:]]*[(]", sep="")
  nefarious = do.call(cbind, lapply(blackList, regexpr, text=code)) # always a matrix
  nefCounts = apply(nefarious, 1, function(x) sum(x!=-1))
  if (sum(nefCounts)>0) {
    errorCode = code[nefCounts>0]
    return(list(input=code, output=c0, errorMessage=c0, errorCode=errorCode, nefarious=TRUE))
  }
  
  p = try(parse(text=code, keep.source=TRUE), silent=TRUE)
  if (is(p, "try-error")) {
    return(list(input=code, output=c0, errorMessage=attr(p, "condition")$message, 
                errorCode=code, nefarious=FALSE))
  }
  pTree = attr(p, "srcref")
  maxRun = ifelse(allowSemicolons, length(pTree), 1)
  codeSplit = lapply(pTree, as.character)[1:maxRun]
  output = character(0)
  for (i in 1:maxRun) {
    pchar = codeSplit[[i]]
    rslt = try(capture.output(eval(parse(text=pchar), envir=env)), silent=TRUE)
    if (is(rslt, "try-error")) {
      return(list(input=code, output=output, errorMessage=attr(rslt, "condition")$message, 
                  errorCode=pchar, nefarious=FALSE))
    }
    output = c(output, list(rslt))
  }
  return(list(input=codeSplit, output=output, errorMessage=c0, errorCode=c0, nefarious=FALSE))
}

